#!/bin/bash
set -e
PROG=$(basename $0 .sh)

# libraries in PREFIX/sysroot
DOCKER_DIR="$(dirname $(realpath $0))"
. ${DOCKER_DIR}/env.sh

TEMP=$(getopt -o hml --long meson,legacy -- "$@")
# Replace arguments by preprocessed arguments
eval set -- "$TEMP"

usage() {
echo "
Name:
  ${PROG}

Options:
 -h
 -m|--meson
 -l|--legacy
    include dsstests in the run

everything after '--' is passed to test programs

Examples:
  meson build, tell test program to abort upon first error
  $ ${PROG} --meson -- -a

  autoconf build, will only run [cacheUpdater] in dss-test other tests are skipped
  $ ${PROG} -- [cacheUpdater]

See also:
  ninja -C build-meson test
"
}

TRUE=0
FALSE=1

LEGACY=$FALSE
BUILD=autoconf
while true; do
    case "$1" in
        --meson|-m) BUILD=meson; shift;;
        --legacy|-l) LEGACY=$TRUE; shift;;
        -h)
          usage
          exit 0;;
        --) shift; break;;
        *) break;;
    esac
done

# export LD_PRELOAD=/lib/x86_64-linux-gnu/libasan.so.5

do_autoconf() {
    echo "ds-test $@"
    ./build-autoconf/build/ds-test $@

    echo "dss-test $@"
    ./build-autoconf/build/dss-test $@
    ./build-autoconf/build/p1-test $@
}

do_meson() {
    echo "ds-test $@"
    ./build-meson/src/ds/ds-test $@

    echo "ds-api-test $@"
    ./build-meson/src/ds-api/ds-api-test $@

    # fails on docker, no daemon running?
    #./build-meson/src/ds-avahi/ds-avahi-test $@

    ./build-meson/src/ds-backend/ds-backend-test $@
    ./build-meson/src/ds-crypt/ds-crypt-test $@
    ./build-meson/src/ds-media-library/ds-media-library-test
    ./build-meson/src/ds-net/ds-net-test
    ./build-meson/src/ds-model/ds-model-test $@
    ./build-meson/src/p1/p1-test $@

    echo "dss-test $@"
    ./build-meson/src/dss-test $@



}

if [ $BUILD = "autoconf" ]; then
    do_autoconf $@
else
    do_meson $@
fi

do_legacy() {
    echo "dsstests"
    # export since running last
    export LSAN_OPTIONS=suppressions=docker/dsstest-suppressions.txt
    if [ $BUILD = "autoconf" ]; then
        # handle SIG32 noprint nostop
        #gdb ./build-autoconf/build/dsstests
        ./build-autoconf/build/dsstests
    else
        # handle SIG32 noprint nostop
        #gdb ./build-meson/src/dsstests
      ./build-meson/src/dsstests
    fi
}

if [ $LEGACY -eq $TRUE ]; then
  do_legacy
fi
