#pragma once

#include <ds-api/rest-fwd.h>
#include <ds-backend/connector-fwd.h>
#include <ds/asio/resource.h>
#include <ds/json.h>

#include <boost/noncopyable.hpp>
#include <memory>

namespace dsBackend {

struct UploaderConfig {
  unsigned capacity = 1000;
};

// Hooks to support testing
class UploaderHooks {
public:
  virtual ~UploaderHooks() = default;
  virtual void nothingToDo() {}
  virtual void blockedOnDisconnected() {}
  virtual void succeeded(const dsApi::HttpRequest&) {}
  virtual void dropped(const dsApi::HttpRequest&) {}
};
/// default instance
extern UploaderHooks uploaderHooks;

/// Queues and uploads records to backed on background
class Uploader : public ds::asio::Resource<UploaderConfig> {
public:
  using ds::asio::Resource<UploaderConfig>::Resource;

  virtual void asyncUpload(dsApi::HttpRequest&& x) = 0;
};
std::unique_ptr<Uploader> makeUploader(ds::asio::ResourceBase&, dsBackend::Connector&,
                                       UploaderHooks& hooks = uploaderHooks);

} // namespace dsBackend

DS_JSON_DECLARE_STRUCT((dsBackend), UploaderConfig, (capacity));
