/*
    Copyright (c) 2018  digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/
#pragma once

#include <trompeloeil.hpp>

#include "signalr.h"

namespace dsNet {

struct SignalrSessionMock : ds::asio::Resource<>, trompeloeil::mock_interface<SignalrSession> {
  using ds::asio::Resource<>::Resource;
  IMPLEMENT_MOCK1(asyncRun);
  IMPLEMENT_MOCK2(invokeNoReply);
  IMPLEMENT_MOCK0(close);
  IMPLEMENT_MOCK0(sharedSession);
};

struct SignalrSessionHooksMock {
  MAKE_MOCK0(opened, void());
  MAKE_MOCK1(closed, void(error_code));
  MAKE_MOCK2(invokeNoReply, void(std::string&& target, std::vector<ds::JsonFragment>&&));
  auto hooks() {
    return SignalrSessionHooks{
        .opened = [this] { opened(); },
        .closed = [this](auto&& x) { closed(DS_FWD(x)); },
        .invokeNoReply = [this](auto&& target, auto&& args) { invokeNoReply(DS_FWD(target), DS_FWD(args)); }};
  }
};

struct SignalrAcceptorMock : public trompeloeil::mock_interface<SignalrAcceptor> {
  IMPLEMENT_MOCK1(asyncRun);
};

struct SignalrAcceptorHooksMock {
  MAKE_MOCK1(accept, void(ClosePtr<SignalrSession>));
  auto hooks() {
    return SignalrAcceptorHooks{.accept = [this](auto&& x) { accept(DS_FWD(x)); }};
  }
};

} // namespace dsNet
