/*
    Copyright (c) 2020 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#pragma once

#include <ds/asio/resource.h>

#include <boost/asio/ip/tcp.hpp>
#include <functional>

namespace dsNet {
using tcp = boost::asio::ip::tcp;

struct Endpoint {
  std::string address{};
  unsigned short port{};

  tcp::endpoint toAsio() const;
};
Endpoint makeEndpoint(const tcp::endpoint&);

struct TcpAcceptorConfig {
  Endpoint endpoint;
};

struct TcpAcceptorHooks {
  std::function<void(tcp::socket&&)> accept;
};

class TcpAcceptor {
public:
  virtual ~TcpAcceptor() = default;

  virtual void asyncRun(TcpAcceptorHooks&&) = 0;
  virtual Endpoint localEndpoint() const = 0;
  virtual tcp::endpoint localAsioEndpoint() const = 0;
};

std::unique_ptr<TcpAcceptor> makeTcpAcceptor(ds::asio::ResourceBase& parent, TcpAcceptorConfig&&);

} // namespace dsNet

DS_JSON_DECLARE_STRUCT((dsNet), Endpoint, (address)(port));
DS_JSON_DECLARE_STRUCT((dsNet), TcpAcceptorConfig, (endpoint));
