#pragma once

#include <utility>

namespace ds::asio {

//
// MoveOnly only handler with declared but undefined copy constructor.
//
// Allows to pass move only handlers to asio post, dispatch calls.
//
// https://stackoverflow.com/questions/17211263/how-to-trick-boostasio-to-allow-move-only-handlers/22891509#22891509

template <typename F>
struct MoveOnly : F {
  MoveOnly(F&& f) : F(std::move(f)) {}

  MoveOnly(MoveOnly&&) = default;
  MoveOnly& operator=(MoveOnly&&) = default;

  MoveOnly(const MoveOnly&);
  MoveOnly& operator=(const MoveOnly&);
};

} // namespace ds::asio
