/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */
#include "thread-pool.h"
#include <ds/log.h>
#include <thread>
#include <vector>

static ds::LogChannel dsLogChannel("dsAsioThreadPool");

namespace ds::asio {

struct ThreadPool::Impl {
  boost::asio::io_service io;
  boost::asio::io_service::work work{io};
  std::vector<std::thread> threads;

  Impl(int concurrency) {
    threads.reserve(concurrency);
    for (int i = 0; i < concurrency; i++) {
      threads.emplace_back([&] {
        for (;;) {
          try {
            io.run();
            break; // run() exited normally
          } catch (std::exception& e) {
            DS_ERROR("Unhandled exception", e);
          }
        }
      });
    }
  }

  ~Impl() {
    io.stop();
    for (auto& thread : threads) {
      thread.join();
    }
  }
};

ThreadPool::ThreadPool(int threads) : m_impl{std::make_unique<Impl>(threads)} {}

ThreadPool::~ThreadPool() = default;

boost::asio::io_service& ThreadPool::io() {
  return m_impl->io;
};

} // namespace ds::asio
