/*
 *  Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#pragma once

#include <ds/common.h>
#include <boost/asio/io_service.hpp>
#include <memory>

namespace ds {
namespace asio {

///
/// Pool of threads executing posted functors.
///
class ThreadPool {
public:
  ThreadPool(int threads);
  ~ThreadPool();

  template <class F>
  void post(F&& f) {
    io().post(std::forward<F>(f));
  }

private:
  struct Impl;
  const std::unique_ptr<Impl> m_impl;

  boost::asio::io_service& io();
};

} // namespace asio
} // namespace ds
