/*
 *  Copyright (c) 2016 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#include "timer.h"
#include <ds/log.h>
#include <ds/random.h>

static ds::LogChannel dsLogChannel("dsAsioTimer");

namespace ds::asio {

Timer::Impl::Impl(boost::asio::io_service &io) : io{io}, component{io} {
  DS_DEBUG_ENTER(this);
}
Timer::Impl::~Impl() {
  DS_DEBUG_ENTER(this);
}

Timer::Timer(boost::asio::io_service &io) : m_impl{std::make_shared<Impl>(io)} {
  DS_DEBUG_ENTER(m_impl.get());
}
Timer::~Timer() {
  DS_DEBUG_ENTER(m_impl.get(), m_impl->seq);
  m_impl->seq++;
}

void Timer::expiresFromNow(duration d) {
  DS_DEBUG_ENTER(m_impl.get(), m_impl->seq, d);
  m_impl->seq++;
  m_impl->component.expires_from_now(d);
}

void Timer::randomlyExpiresFromNow(duration a, duration b) {
  expiresFromNow(duration(ds::randint(a.count(), b.count())));
}

void Timer::randomlyExpiresFromNowPercentDown(duration d, int p) {
  DS_REQUIRE(p > 0 && p < 100, p);
  randomlyExpiresFromNow(d - d / 100 * p, d);
}

void Timer::randomlyExpiresFromNow(duration d) {
  randomlyExpiresFromNow(d, d * 11 / 10);
}

void Timer::cancel() {
  DS_DEBUG_ENTER(m_impl.get(), m_impl->seq);
  m_impl->seq++;
  m_impl->component.cancel();
}

} // namespace ds::asio
