/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/
#include <gsl.h>
#include <array>
#include "common.h"

namespace ds {

struct StackTrace {
  std::array<void*, 64> buffer{};
  int first{};
  int afterLast{};

  gsl::span<void*> span() const {
    return gsl::make_span(const_cast<void**>(buffer.data() + first), const_cast<void**>(buffer.data() + afterLast));
  }

  /// Create stack trace with current stack
  static StackTrace capture(int ignoreCount = 0);
};

std::ostream& operator<<(std::ostream& stream, const StackTrace& x);

/// Write stack trace symbols directly to file descriptor.
/// Useful from asserts ... to avoids ostream <<
void writeSymbolsToFd(const StackTrace&, int fd);

} // namespace ds
