/*
 *  Copyright (c) 2015 digitalSTROM.org, Zurich, Switzerland
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _EVENT_CREATE_H_
#define _EVENT_CREATE_H_

#include "event.h"

#include <memory>
#include <digitalSTROM/dsuid.h>

#include "model/modelconst.h"
#include "model/data_types.h"
#include "model/deviceinterface.h"
#include "model/zone.h"

namespace vdcapi {
class PropertyElement;
class Message;
} // namespace vdcapi

namespace dss {

std::shared_ptr<Event>
  createDeviceStatusEvent(std::shared_ptr<DeviceReference> _devRef,
                          int _index, int _value);

std::shared_ptr<Event>
  createDeviceBinaryInputEvent(std::shared_ptr<DeviceReference> _devRef,
                               int _index, BinaryInputType _type, BinaryInputStateValue _state);

std::shared_ptr<Event>
createDeviceCustomActionChangedEvent(std::shared_ptr<DeviceReference> _devRef,
    const std::string& id, const std::string& action, const std::string& title,
    const vdcapi::PropertyElement& params);

///< Will emits same message as CustomActionChanged but with different fields
std::shared_ptr<Event>
createDeviceActionChangedEvent(std::shared_ptr<DeviceReference> _devRef,
    const std::string& name, const std::string& title);

std::shared_ptr<Event>
createDeviceActionEvent(std::shared_ptr<DeviceReference> _devRef, const std::string& name,
    const vdcapi::PropertyElement& params);

std::shared_ptr<Event>
  createDeviceStateEvent(std::shared_ptr<DeviceReference> _devRef,
                             const std::string& name, const std::string& value);
std::shared_ptr<Event>
  createDeviceEventEvent(std::shared_ptr<DeviceReference> _devRef,
                             const std::string& name);
std::shared_ptr<Event>
  createDeviceSensorValueEvent(std::shared_ptr<DeviceReference> _devRef,
                               int _index, SensorType _type, int _value);

std::shared_ptr<Event>
  createDeviceSensorValueExEvent(std::shared_ptr<DeviceReference> _devRef,
                             int _index, SensorType _type, double _value, uint32_t _age,
                             uint32_t _contextId, const std::string& _contextMsg);

std::shared_ptr<Event>
  createDeviceInvalidSensorEvent(std::shared_ptr<DeviceReference> _devRef,
                                 int _index, SensorType _type, const DateTime& _ts);

std::shared_ptr<Event>
  createZoneSensorValueEvent(std::shared_ptr<Group> _group, SensorType _type,
                             int _value, const dsuid_t &_sourceDevice);

std::shared_ptr<Event>
  createZoneSensorErrorEvent(std::shared_ptr<Group> _group, SensorType _type,
                             const DateTime& _ts);

std::shared_ptr<Event>
  createGroupCallSceneEvent(std::shared_ptr<Group> _group, int _sceneID,
                            int _groupID, int _zoneID,
                            const CallOrigin& _callOrigin,
                            const dsuid_t& _originDSUID,
                            const std::string& _originToken,
                            bool _forced);

std::shared_ptr<Event>
  createGroupUndoSceneEvent(std::shared_ptr<Group> _group, int _sceneID,
                            int _groupID, int _zoneID,
                            const CallOrigin& _callOrigin,
                            const dsuid_t& _originDSUID,
                            const std::string& _originToken);

std::shared_ptr<Event>
  createStateChangeEvent(std::shared_ptr<State> _state, int _oldstate,
                         CallOrigin _callOrigin);

std::shared_ptr<Event>
  createActionDenied(const std::string &_type, const std::string &_name,
                     const std::string &_source, const std::string &_reason);

std::shared_ptr<Event>
  createHeatingEnabled(bool _heatingEnabled, bool _coolingEnabled);

std::shared_ptr<Event>
  createHeatingSystemCapability(bool _heatingSupported, bool _coolingSupported);

std::shared_ptr<Event>
  createHeatingControllerConfig(int _zoneID, const dsuid_t &_ctrlDsuid,
                              const ZoneHeatingConfigSpec_t &_config);

std::shared_ptr<Event> createHeatingControllerValue(int _zoneID, const dsuid_t& _ctrlDsuid,
    const TemperatureConfig& _properties, const ZoneHeatingOperationModeSpec_t& _mode);

std::shared_ptr<Event>
  createHeatingControllerState(int _zoneID, const dsuid_t &_ctrlDsuid, int _ctrlState);

std::shared_ptr<Event>
  createOldStateChange(const std::string &_scriptId, const std::string &_name,
                       const std::string &_value, CallOrigin _origin);

std::shared_ptr<Event>
  createGenericSignalSunshine(const uint8_t &_value, const CardinalDirection_t &_direction, CallOrigin _origin);

std::shared_ptr<Event>
  createGenericSignalFrostProtection(const uint8_t &_value, CallOrigin _origin);

std::shared_ptr<Event>
  createGenericSignalHeatingModeSwitch(HeatingModeSwitchValue value, CallOrigin _origin);

std::shared_ptr<Event>
  createGenericSignalBuildingService(const uint8_t &_value, CallOrigin _origin);

std::shared_ptr<Event>
  createOperationLockEvent(std::shared_ptr<Group> _group,
      const int _zoneID, const int _groupID, const bool _lock, CallOrigin _origin);

std::shared_ptr<Event>
  createClusterConfigLockEvent(std::shared_ptr<Group> _group, const int _zoneID, const int _groupID,
      const bool _lock, CallOrigin _origin);

std::shared_ptr<Event>
  createClusterCheckWindEvent(const int _clusterID, const int time);
}
#endif
