/*
    Copyright (c) 2009,2010,2011 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.

*/

#pragma once

#include <ds/system-fwd.h>

#include <filesystem>

#include "archives-fwd.h"
#include "metering/archives/modbusmeteringsystem.h"

namespace dss::metering {

// ArchiveId is already taken
using CompoundId = std::variant<std::monostate, ArchiveId, OpenMeteringSystemId, ModbusMeteringId, dsid_t>;

std::string filename(const ArchiveId& archiveId);

class ArchivesStorage {
public:
  virtual ~ArchivesStorage() = default;

  virtual void setMeteringStorageLocation(std::string&& meteringStorageLocation) = 0;

  virtual std::string filepath(const ArchiveId& archiveId) const = 0;
  virtual bool hasArchive(const ArchiveId& archiveId) const = 0;

  virtual std::string filepath(const OpenMeteringSystemId& openMeteringSystemId) const = 0;
  virtual bool hasArchive(const OpenMeteringSystemId& openMeteringSystemId) const = 0;
  virtual std::vector<OpenMeteringSystemId> searchOmsArchives() const = 0;

  virtual std::string filepath(const archives::ModbusMeteringArchiveId& modbusMeteringId) const = 0;
  virtual bool hasArchive(const archives::ModbusMeteringArchiveId& modbusMeteringId) const = 0;

  virtual std::vector<std::filesystem::path> searchRrdFiles() const = 0;
  virtual std::vector<std::pair<CompoundId, std::string>> searchArchives() const = 0;

  // transitional, hide later
  virtual std::string storageLocation() const = 0;
};

std::unique_ptr<ArchivesStorage> makeArchivesStorage(ds::System& system);

} // namespace dss::metering
