#pragma once

#include <ds/enum.h>
#include <rrd.h>

#include <ostream>
#include <vector>

#include "metering.h"

DS_REFLECTED_ENUM_WITH_INITIALIZERS((dss), RrdInfoType,
                                    ((value)(0))((counter)(1))((string)(2))((integer)(3))((blob)(4)));
namespace dss::metering {

using counter_t = unsigned long;
using blob_t = std::vector<unsigned char>;
using RrdInfoValue = std::variant<counter_t, rrd_value_t, std::string, int, blob_t>;

struct RRDInfo {
  std::string key;
  RrdInfoType type;
  RrdInfoValue value;
};

class RRDInterface {
public:
  virtual ~RRDInterface() = default;
  virtual ValueRange xport(std::vector<std::string> args) = 0;
  virtual void create(std::string filename, unsigned long pdpStep, time_t lastUpdate,
                      std::vector<std::string> args) = 0;
  virtual std::vector<RRDInfo> info(std::string filename) = 0;
  virtual void tune(std::vector<std::string> arguments) = 0;
  virtual void update(std::vector<std::string> arguments) = 0;
  virtual void flush(std::vector<std::string> arguments) = 0;
  virtual std::pair<time_t, std::vector<std::pair<std::string, double>>> lastValue(const std::string& filename) = 0;
  virtual std::optional<time_t> lastTimestamp(std::string filename) = 0;
};

std::shared_ptr<RRDInterface> makeRRDInterface();

} // namespace dss::metering

DS_REFLECT_STRUCT((dss::metering), RRDInfo, (key)(type)(value));
