/*
    Copyright (c) 2021 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#pragma once

#include <memory>

#include "dssfwd.h"

namespace dss {

namespace model {
struct ShadeAttributes;
};

struct DevicePropertyTree {
  virtual ~DevicePropertyTree() = default;
  virtual void publishToPropertyTree(Device& device) = 0;
  virtual void publishVdcToPropertyTree(Device& device) = 0;
  virtual void publishValveTypeToPropertyTree(Device& device) = 0;
  virtual void publishAliasInZoneGroups(const Device& device) const = 0;
  virtual void publishDeviceSpecification(Device& device) const = 0;
  virtual void republishModelFeaturesToPropertyTree(Device& device) = 0;
  virtual void updateAKMNode(Device& device) = 0;
  virtual void removeFromPropertyTree(Device& device) = 0;
  virtual void publishShadeAttributes(PropertyNode& deviceRoot, model::ShadeAttributes& shadeAttributes) = 0;
  virtual void removeShadeAttributes(PropertyNode& deviceRoot) = 0;

  virtual void publishGroupNode(Device& device, int groupId) = 0;
  virtual void removeGroupNode(Device& device, int groupId) = 0;
};

std::unique_ptr<DevicePropertyTree> makeDevicePropertyTree(Apartment& apartment);

} // namespace dss
