/*
    Copyright (c) 2019 digitalSTROM.org, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation," " either version 3 of the License," " or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful," "
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not," " see <http://www.gnu.org/licenses/>.
*/

#include "model/deviceinterface.h"

#include <ds/catch/catch.h>
#include <ds/log.h>

static ds::LogChannel dsLogChannel("modelDeviceInterfaceTest");
static const char* TAGS = "[model][deviceinterface]";

using namespace dss;
using namespace std::literals;
using namespace trompeloeil;
using trompeloeil::_;

namespace {

TEST_CASE("DeviceInterface", TAGS) {
  SECTION("CallOrigin") {
    SECTION("domain") {
      struct X {
        CallOrigin callOrigin;
        int value;
      };
      const auto& x = GENERATE(values(std::initializer_list<X>{{CallOrigin::unknown, 0},
                                                               {CallOrigin::scripting, 1},
                                                               {CallOrigin::json, 2},
                                                               {CallOrigin::subscription, 4},
                                                               {CallOrigin::test, 6},
                                                               {CallOrigin::system, 7},
                                                               {CallOrigin::binaryInput, 8},
                                                               {CallOrigin::dsmApi, 9},
                                                               {CallOrigin::systemStartup, 10}}));
      INFO(ds::str(x.value));
      CHECK(static_cast<int>(x.callOrigin) == x.value);
      CHECK(ds::isValidEnum(x.callOrigin));
    }
    SECTION("invalid") {
      const auto& x = GENERATE(3, 5, 11);
      INFO(ds::str(x));
      CHECK(!ds::isValidEnum(static_cast<CallOrigin>(x)));
    }
    SECTION("default constructor") {
      CHECK(CallOrigin{} == CallOrigin::unknown);
      CHECK(ds::str(CallOrigin{}) == "unknown"s);
      CHECK(ds::str(static_cast<CallOrigin>(1001)) == ""s); // 1001 will not show in log messages
      CHECK(ds::toEnum<CallOrigin>(1001) == CallOrigin::unknown);
    }
  }
}
}
