/*
    Copyright (c) 2010,2011 digitalSTROM.org, Zurich, Switzerland
    Copyright (c) 2022 Aizo AG, Zurich, Switzerland

    This file is part of digitalSTROM Server.

    digitalSTROM Server is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    digitalSTROM Server is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#include "databasedownload.h"

#include <ds/log.h>

#include "dss.h"
#include "event.h"
#include "http_client.h"
#include "sqlite3_wrapper.h"

static ds::LogChannel dsLogChannel("scannerDatabaseDownload");

namespace dss {
DatabaseDownload::DatabaseDownload(std::string _script_id, std::string _url)
    : Task(), m_scriptId(_script_id), m_url(_url) {}

void DatabaseDownload::run() {
  if (m_scriptId.empty() || m_url.empty()) {
    DS_ERROR("DatabaseDownload: invalid configuration, missing parameters");
    return;
  }

  std::shared_ptr<Event> pEvent = std::make_shared<Event>(EventName::DatabaseImported);

  pEvent->setProperty("id", m_scriptId);
  pEvent->setProperty("url", m_url);
  try {
    HttpClient http;
    std::string result;

    long req = http.get(m_url, &result);
    if (req != 200) {
      throw std::runtime_error("Could not download database from " + m_url + ": HTTP code " + intToString(req));
    }

    std::string database = DSS::getInstance()->getDatabaseDirectory() + m_scriptId + ".db";
    if (!result.empty()) {
      SQLite3 sqlite(database, SQLite3::Mode::ReadWrite);
      sqlite.exec(result);
      pEvent->setProperty("success", "1");
      DSS::getInstance()->getEventQueue().pushEvent(pEvent);
    }
  } catch (std::runtime_error &ex) {
    DS_ERROR("Exception when importing database:", ex.what());
    pEvent->setProperty("success", "0");
    DSS::getInstance()->getEventQueue().pushEvent(pEvent);
  }
}

} // namespace dss
