/*
 *  Copyright (c) 2009 digitalSTROM.org, Zurich, Switzerland
 *
 *  Author: Andreas Fenkart, <andreas.fenkart@digitalstrom.com>
 *
 *  This file is part of digitalSTROM Server.
 *
 *  digitalSTROM Server is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  digitalSTROM Server is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
 *
 */

#define BOOST_TEST_NO_MAIN
#include <boost/test/unit_test.hpp>

#include <boost/filesystem.hpp>
#include <boost/scoped_ptr.hpp>
#include <fstream>
#include <iostream>

#include "ds485types.h"
#include "model/device.h"
#include "model/apartment.h"
#include "model/dsmeter.h"
#include "model/devicereference.h"
#include "model/zone.h"
#include "model/group.h"
#include "model/cluster.h"
#include "model/set.h"
#include "model/modelpersistence.h"
#include "setbuilder.h"
#include "dss.h"
#include "model/modelconst.h"
#include "model/modelmaintenance.h"
#include "propertysystem.h"
#include "structuremanipulator.h"

using namespace dss;

BOOST_AUTO_TEST_SUITE(apartment_xml)

BOOST_AUTO_TEST_CASE(testApartmentXML) {
  PropertySystem propSys;
  auto pApartment = makeApartment(nullptr, &propSys);
  auto& apt = *pApartment;

  ModelPersistence model(apt);

  /*
   * Current apartment xml format,
   * Verify that we can still read it
   */

  std::string fileName = getTempDir() + "/apt.xml";
  std::string invalidBackup = getTempDir() + "/invalid.xml";
  std::ofstream ofs(fileName.c_str());
  ofs <<
    "<?xml version=\"1.0\" encoding=\"utf-8\"?>\n"
    "<config version=\"1\">\n"
    "  <apartment>\n"
    "    <name>UAZ</name>\n"
    "  </apartment>\n"
    "  <devices>\n"
    "    <device dsid=\"3504175fe0000000ffc00024\" firstSeen=\"1327676779\" isPresent=\"1\" lastKnownDSMeter=\"3504175fe0000000ffc00010\" lastKnownShortAddress=\"24\" lastKnownZoneID=\"1\">\n"
    "      <name>UAZ-469</name>\n"
    "      <properties>\n"
    "        <property archive=\"true\" name=\"my/custom/setting\" readable=\"true\" type=\"integer\" writeable=\"true\">\n"
    "          <value>31512</value>\n"
    "        </property>\n"
    "      </properties>\n"
    "    </device>\n"
    "    <device dsuid=\"0000000000000000000000000000000001\" isPresent=\"0\" firstSeen=\"1429729274\" inactiveSince=\"1970-01-01 01:00:00\" CardinalDirection=\"north west\" WindProtectionClass=\"6\" configurationLocked=\"0\" Floor=\"13\">\n"
    "    </device>\n"
    "  </devices>\n"
    "  <zones>\n"
    "    <zone id=\"0\">\n"
    "      <groups>\n"
    "        <group id=\"16\">\n"
    "          <name>Jalousien</name>\n"
    "          <color>2</color>\n"
    "          <scenes>\n"
    "          </scenes>\n"
    "        </group>\n"
    "      </groups>\n"
    "    </zone>\n"
    "    <zone id=\"1\">\n"
    "      <name>garazh</name>\n"
    "      <groups>\n"
    "        <group id=\"1\">\n"
    "          <name>yellow</name>\n"
    "          <color>1</color>\n"
    "          <configuration>0x0</configuration>\n"
    "          <scenes>\n"
    "             <scene id=\"5\">\n"
    "               <name>T0_S1</name>\n"
    "             </scene>\n"
    "             <scene id=\"6\">\n"
    "               <name>T1_S4</name>\n"
    "             </scene>\n"
    "             <scene id=\"11\">\n"
    "               <name>DEC</name>\n"
    "             </scene>\n"
    "             <scene id=\"12\">\n"
    "               <name>INC</name>\n"
    "             </scene>\n"
    "             <scene id=\"13\">\n"
    "               <name>MIN</name>\n"
    "             </scene>\n"
    "             <scene id=\"14\">\n"
    "               <name>MAX</name>\n"
    "             </scene>\n"
    "             <scene id=\"15\">\n"
    "               <name>STOP</name>\n"
    "             </scene>\n"
    "             <scene id=\"50\">\n"
    "               <name>LOCAL_ON</name>\n"
    "             </scene>\n"
    "             <scene id=\"51\">\n"
    "               <name>LOCAL_OFF</name>\n"
    "             </scene>\n"
    "             <scene id=\"64\">\n"
    "               <name>AUTO_STANDBY</name>\n"
    "             </scene>\n"
    "             <scene id=\"65\">\n"
    "               <name>SIG_PANIC</name>\n"
    "             </scene>\n"
    "             <scene id=\"66\">\n"
    "               <name>ENERGY_OL</name>\n"
    "             </scene>\n"
    "             <scene id=\"67\">\n"
    "               <name>STANDBY</name>\n"
    "             </scene>\n"
    "             <scene id=\"68\">\n"
    "               <name>DEEP_OFF</name>\n"
    "             </scene>\n"
    "             <scene id=\"69\">\n"
    "               <name>SLEEPING</name>\n"
    "             </scene>\n"
    "             <scene id=\"70\">\n"
    "               <name>WAKE_UP</name>\n"
    "             </scene>\n"
    "             <scene id=\"71\">\n"
    "               <name>PRESENT</name>\n"
    "             </scene>\n"
    "             <scene id=\"72\">\n"
    "               <name>ABSENT</name>\n"
    "             </scene>\n"
    "             <scene id=\"73\">\n"
    "               <name>SIG_BELL</name>\n"
    "             </scene>\n"
    "             <scene id=\"74\">\n"
    "               <name>SIG_ALARM</name>\n"
    "             </scene>\n"
    "          </scenes>\n"
    "        </group>\n"
    "        <group id=\"2\">\n"
    "          <name>grey</name>\n"
    "          <color>2</color>\n"
    "          <configuration>0x0</configuration>\n"
    "          <scenes>\n"
    "             <scene id=\"9\">\n"
    "               <name>t0_S1</name>\n"
    "             </scene>\n"
    "             <scene id=\"10\">\n"
    "               <name>T1_s4</name>\n"
    "             </scene>\n"
    "             <scene id=\"7\">\n"
    "               <name>T1_S5</name>\n"
    "             </scene>\n"
    "             <scene id=\"8\">\n"
    "               <name>T5_S0</name>\n"
    "             </scene>\n"
    "             <scene id=\"11\">\n"
    "               <name>dEC</name>\n"
    "             </scene>\n"
    "             <scene id=\"12\">\n"
    "               <name>INc</name>\n"
    "             </scene>\n"
    "             <scene id=\"13\">\n"
    "               <name>MiN</name>\n"
    "             </scene>\n"
    "             <scene id=\"14\">\n"
    "               <name>MAx</name>\n"
    "             </scene>\n"
    "             <scene id=\"15\">\n"
    "               <name>STOP </name>\n"
    "             </scene>\n"
    "             <scene id=\"16\">\n"
    "               <name>LOCALON</name>\n"
    "             </scene>\n"
    "             <scene id=\"17\">\n"
    "               <name> LOCAL_OFF</name>\n"
    "             </scene>\n"
    "             <scene id=\"18\">\n"
    "               <name>AUTO%STANDBY</name>\n"
    "             </scene>\n"
    "             <scene id=\"19\">\n"
    "               <name>SIG_PANiC</name>\n"
    "             </scene>\n"
    "             <scene id=\"20\">\n"
    "               <name>ENERgY_OL</name>\n"
    "             </scene>\n"
    "             <scene id=\"21\">\n"
    "               <name>STAnDBY</name>\n"
    "             </scene>\n"
    "             <scene id=\"22\">\n"
    "               <name>dEEP_OFF</name>\n"
    "             </scene>\n"
    "             <scene id=\"23\">\n"
    "               <name>SLEEpING</name>\n"
    "             </scene>\n"
    "             <scene id=\"24\">\n"
    "               <name>WAKE_uP</name>\n"
    "             </scene>\n"
    "             <scene id=\"25\">\n"
    "               <name>PRESENt</name>\n"
    "             </scene>\n"
    "             <scene id=\"26\">\n"
    "               <name>ABS ENT</name>\n"
    "             </scene>\n"
    "             <scene id=\"27\">\n"
    "               <name>hihi</name>\n"
    "             </scene>\n"
    "             <scene id=\"28\">\n"
    "               <name>Huhu</name>\n"
    "             </scene>\n"
    "          </scenes>\n"
    "        </group>\n"
    "      </groups>\n"
    "    </zone>\n"
    "  </zones>\n"
    "  <clusters>\n"
    "    <cluster id=\"17\">\n"
    "      <name>Jalousien2</name>\n"
    "      <color>2</color>\n"
    "      <location>south east</location>\n"
    "      <protectionClass>2</protectionClass>\n"
    "      <floor>1</floor>\n"
    "      <configurationLocked>0</configurationLocked>\n"
    "      <automatic>1</automatic>\n"
    "      <lockedScenes>\n"
    "        <lockedScene id=\"5\" />\n"
    "        <lockedScene id=\"17\" />\n"
    "        <lockedScene id=\"18\" />\n"
    "        <lockedScene id=\"19\" />\n"
    "      </lockedScenes>\n"
    "    </cluster>\n"
    "  </clusters>\n"
    "  <dsMeters>\n"
    "    <dsMeter id=\"3504175fe0000000ffc00010\">\n"
    "      <name>UMZ-451</name>\n"
    "      <datamodelHash>559038737</datamodelHash>\n"
    "      <datamodelModification>0</datamodelModification>\n"
    "    </dsMeter>\n"
    "  </dsMeters>\n"
    "</config>";

  ofs.close();

  Logger::getInstance()->log("Checking apartment XML");

  model.readConfigurationFromXML(fileName, invalidBackup);
  BOOST_CHECK_EQUAL(apt.getName(), "UAZ");

  std::shared_ptr<Device> dev = apt.getDeviceByName("UAZ-469");
  BOOST_CHECK(dev != NULL);
  BOOST_CHECK_EQUAL(dev->getCardinalDirection(), cd_none);
  BOOST_CHECK_EQUAL(dev->getWindProtectionClass(), wpc_none);
  BOOST_CHECK_EQUAL(dev->getFloor(), 0);

  std::shared_ptr<DSMeter> meter = apt.getDSMeter("UMZ-451");
  BOOST_CHECK(meter != NULL);

  std::shared_ptr<Zone> zone = apt.getZoneByName("garazh");
  BOOST_CHECK(zone != NULL);

  for (int i = 0; i < 128; i++) {
    BOOST_CHECK_EQUAL(zone->getGroup(1)->getSceneName(i), "");
  }

  for (int i = 7; i < 29; i++) {
    BOOST_CHECK_NE(zone->getGroup(2)->getSceneName(i), "");
  }

  BOOST_CHECK_EQUAL(dev->getZoneID(), 1);
  BOOST_CHECK_EQUAL(zone->getID(), 1);

  PropertyNodePtr prop = dev->getPropertyNode();
  BOOST_CHECK(prop != NULL);
  BOOST_CHECK_EQUAL(prop->getProperty("my/custom/setting")->getIntegerValue(),
                    31512);

  /* device with protection/direction */
  DSUID_DEFINE(protID, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1);
  dev = apt.getDeviceByDSID(protID);
  BOOST_CHECK_EQUAL(dev->getCardinalDirection(), cd_north_west);
  BOOST_CHECK_EQUAL(dev->getWindProtectionClass(), wpc_blind_class_3);
  BOOST_CHECK_EQUAL(dev->getFloor(), 13);

  std::shared_ptr<Cluster> pClust;
  BOOST_CHECK_NO_THROW(pClust = apt.getCluster(16));
  BOOST_CHECK_EQUAL(pClust->getName(), "Jalousien");
  BOOST_CHECK_EQUAL(pClust->getApplicationType(), ApplicationType::blinds);

  BOOST_CHECK_NO_THROW(pClust = apt.getCluster(17));
  BOOST_CHECK_EQUAL(pClust->getName(), "Jalousien2");
  BOOST_CHECK_EQUAL(pClust->getApplicationType(), ApplicationType::blinds);
  BOOST_CHECK_EQUAL(pClust->getLocation(), cd_south_east);
  BOOST_CHECK_EQUAL(pClust->getProtectionClass(), wpc_awning_class_2);
  BOOST_CHECK_EQUAL(pClust->getFloor(), 1);
  BOOST_CHECK_EQUAL(pClust->getApplicationType(), ApplicationType::blinds);
  BOOST_CHECK_EQUAL(pClust->isConfigurationLocked(), false);
  BOOST_CHECK_EQUAL(pClust->isAutomatic(), true);
  BOOST_CHECK_EQUAL(pClust->getLockedScenes().size(), 4);

  boost::filesystem::remove_all(fileName);
}

BOOST_AUTO_TEST_CASE(testPersistCardinalDirection) {
  char *dirname, tmpl[] = "/tmp/dss‐persistence-test_XXXXXX";
  std::string filename;

  dirname = mkdtemp(tmpl);
  if (dirname == NULL) {
    BOOST_TEST_MESSAGE("Failed to create temporary folder\n");
  }

  {
    DeviceSpec spec = {};
    spec.FunctionID = 0x2131;
    spec.ProductID = ProductID_KL_210;

    // create apartment xml
    auto pApartment = makeApartment(nullptr);
    auto& apt1 = *pApartment;
    std::shared_ptr<Device> dev1 = apt1.allocateDevice(DSUID_NULL);
    dev1->setPartiallyFromSpec(spec);
    dev1->setCardinalDirection(cd_west);
    dev1->setWindProtectionClass(wpc_awning_class_2);
    dev1->setFloor(42);

    // 2nd device without protecton calls, orientation
    std::shared_ptr<Device> dev2 = apt1.allocateDevice(DSUID_BROADCAST);

    ModelPersistence persist1(apt1);
    filename = std::string(dirname) + "/orientation_protection.xml";
    persist1.writeConfigurationToXML(filename);
  }

  auto pApartment = makeApartment(nullptr);
  auto& apt2 = *pApartment;
  ModelPersistence persist2(apt2);
  persist2.readConfigurationFromXML(filename, "");

  BOOST_CHECK_NO_THROW(apt2.getDeviceByDSID(DSUID_NULL));
  std::shared_ptr<Device> dev = apt2.getDeviceByDSID(DSUID_NULL);

  BOOST_CHECK_EQUAL(dev->getCardinalDirection(), cd_west);
  BOOST_CHECK_EQUAL(dev->getWindProtectionClass(), wpc_awning_class_2);
  BOOST_CHECK_EQUAL(dev->getFloor(), 42);

  BOOST_CHECK_NO_THROW(apt2.getDeviceByDSID(DSUID_BROADCAST));
  dev = apt2.getDeviceByDSID(DSUID_BROADCAST);

  BOOST_CHECK_EQUAL(dev->getCardinalDirection(), cd_none);
  BOOST_CHECK_EQUAL(dev->getWindProtectionClass(), wpc_none);
  BOOST_CHECK_EQUAL(dev->getFloor(), 0);

  unlink(filename.c_str());
  rmdir(dirname);
}

DSUID_DEFINE(dsuid1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 10);
DSUID_DEFINE(dsuid2, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1);

BOOST_AUTO_TEST_CASE(testPersistVdcDevice) {
  char *dirname, tmpl[] = "/tmp/dss‐persistence-test_VDC_XXXXXX";
  std::string filename;

  dirname = mkdtemp(tmpl);
  if (dirname == NULL) {
    BOOST_TEST_MESSAGE("Failed to create temporary folder\n");
  }

  {
    DeviceSpec spec = {};
    spec.FunctionID = 0x2131;
    spec.ProductID = ProductID_KL_210;

    // create apartment xml
    auto pApartment = makeApartment(nullptr);
    auto& apt1 = *pApartment;
    std::shared_ptr<Device> dev1 = apt1.allocateDevice(dsuid1);
    dev1->setPartiallyFromSpec(spec);
    dev1->setVdcDevice(true);

    // 2nd device without protecton calls, orientation
    std::shared_ptr<Device> dev2 = apt1.allocateDevice(dsuid2);
    dev2->setPartiallyFromSpec(spec);
    dev2->setVdcDevice(false);

    ModelPersistence persist1(apt1);
    filename = std::string(dirname) + "/vdc_device.xml";
    persist1.writeConfigurationToXML(filename);
  }

  auto pApartment = makeApartment(nullptr);
  auto& apt2 = *pApartment;
  ModelPersistence persist2(apt2);
  persist2.readConfigurationFromXML(filename, "");

  BOOST_CHECK_NO_THROW(apt2.getDeviceByDSID(dsuid1));
  std::shared_ptr<Device> dev1 = apt2.getDeviceByDSID(dsuid1);

  BOOST_CHECK_EQUAL(dev1->isVdcDevice(), true);

  BOOST_CHECK_NO_THROW(apt2.getDeviceByDSID(dsuid2));
  std::shared_ptr<Device> dev2 = apt2.getDeviceByDSID(dsuid2);

  BOOST_CHECK_EQUAL(dev2->isVdcDevice(), false);

  unlink(filename.c_str());
  rmdir(dirname);
}


BOOST_AUTO_TEST_CASE(testCluster) {
  char *dirname, tmpl[] = "/tmp/dss‐persistence-test_XXXXXX";
  std::string filename;

  dirname = mkdtemp(tmpl);
  if (dirname == NULL) {
    BOOST_TEST_MESSAGE("Failed to create temporary folder\n");
  }

  {
    // create apartment xml
    auto pApartment = makeApartment(nullptr);
    auto& apt1 = *pApartment;
    std::shared_ptr<Cluster> pCluster;
    std::shared_ptr<Zone> zoneBroadcast = apt1.getZone(0);
    pCluster.reset(new Cluster(39, apt1));
    zoneBroadcast->addGroup(pCluster);

    pCluster->setName("Testing");
    pCluster->setApplicationType(ApplicationType::audio);
    pCluster->setLocation(cd_east);
    pCluster->setProtectionClass(wpc_blind_class_3);
    pCluster->setFloor(13);
    pCluster->setConfigurationLocked(true);
    pCluster->setAutomatic(true);
    std::vector<int> scenes;
    scenes.push_back(5);
    scenes.push_back(43);
    scenes.push_back(127);
    pCluster->setLockedScenes(scenes);

    ModelPersistence persist1(apt1);
    filename = std::string(dirname) + "/cluster.xml";
    persist1.writeConfigurationToXML(filename);
  }

  auto pApartment = makeApartment(nullptr);
  auto& apt2 = *pApartment;
  ModelPersistence persist2(apt2);
  persist2.readConfigurationFromXML(filename, "");

  std::shared_ptr<Cluster> pClust;
  BOOST_CHECK_NO_THROW(pClust = apt2.getCluster(39));
  BOOST_CHECK_EQUAL(pClust->getName(), "Testing");
  BOOST_CHECK_EQUAL(pClust->getApplicationType(), ApplicationType::audio);
  BOOST_CHECK_EQUAL(pClust->getLocation(), cd_east);
  BOOST_CHECK_EQUAL(pClust->getProtectionClass(), wpc_blind_class_3);
  BOOST_CHECK_EQUAL(pClust->getFloor(), 13);
  BOOST_CHECK_EQUAL(pClust->isConfigurationLocked(), true);
  BOOST_CHECK_EQUAL(pClust->isAutomatic(), true);
  BOOST_CHECK_EQUAL(pClust->getLockedScenes().size(), 3);

  unlink(filename.c_str());
  rmdir(dirname);
}

BOOST_AUTO_TEST_SUITE_END()
